
import javax.swing.JPanel;
import java.awt.event.*;
import java.awt.event.MouseAdapter;
import java.awt.*;
/**Se implemento lo necesario para que llame a la eliminacion de un objeto al hacer click en el.*/
public class MouseListener extends MouseAdapter {

    private MyWorld world;
    private Drawable currentElement;
    private JPanel panel;

    public Drawable getCurrent() {
        return currentElement;
    }

    public MouseListener(MyWorld w, JPanel p) {
        world = w;
        panel = p;
    }

    public void mouseMoved(MouseEvent e) {
        Drawable newElement = world.find(e.getX(), e.getY());
        if (newElement == currentElement) {
            return;
        }
        if (currentElement != null) {
            currentElement.setReleased();
            currentElement = null;
        }
        if (newElement != null) {
            currentElement = newElement;
            currentElement.setSelected();
        }
        panel.repaint();
    }

    public void mouseDragged(MouseEvent e) {
        if (currentElement == null) {
            return;
        }
        currentElement.dragTo(e.getX(), e.getY());
        panel.repaint();
    }
/**Si world.isRemoving() retorna verdadero entonces al hacer click borrara el objeto, caso contrario actuara como en la version anterior.*/
    public void mouseReleased(MouseEvent e) {
        if (!world.isRemoving()) {

            if (currentElement != null) {
                currentElement.setReleased();
            }
            if (!(currentElement instanceof GSpring) && !(currentElement instanceof GRubber)) {
                currentElement = null;
                return;
            }
            // we dragged an attachable element
            // we dragged a spring, so we look for and attachable element near by  
            AttachableElement element = world.findAttachableElement(e.getX(), e.getY());
            if (element == null) {
                currentElement = null;
                return;
            }

            if (currentElement instanceof GSpring) {

                // we dragged a spring and it is near an attachable element,
                // so we hook them to the right spring end.
                GSpring spring = (GSpring) currentElement;
                Vector2D a = spring.getAendPosition();
                if ((a.getX() == e.getX()) && (a.getY() == e.getY())) {
                    spring.attachAend(element);
                }
                Vector2D b = spring.getBendPosition();
                if ((b.getX() == e.getX()) && (b.getY() == e.getY())) {
                    spring.attachBend(element);
                }
            } else if (currentElement instanceof GRubber) {
                // we dragged a rubber and it is near an attachable element,
                // so we hook them to the right rubber end.
                GRubber rubber = (GRubber) currentElement;
                Vector2D a = rubber.getAendPosition();
                if ((a.getX() == e.getX()) && (a.getY() == e.getY())) {
                    rubber.attachAend(element);
                }
                Vector2D b = rubber.getBendPosition();
                if ((b.getX() == e.getX()) && (b.getY() == e.getY())) {
                    rubber.attachBend(element);
                }
            }
            currentElement = null;
            panel.repaint();
        } else {
            Drawable element = world.find(e.getX(), e.getY());

            if (element != null) {
                world.removeElement(element);
                panel.repaint();
            }
        }

    }
}
