
import javax.swing.Timer;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Graphics2D;
import java.util.*;
/**A esta clase se le agrego el metodo isRunning(),Start(), Resume(), toggleRemoveElement() y isRemoving() y removeElement()*/
public class MyWorld implements ActionListener {

    public static int WIDTH = 600;  // in pixels
    public static int HEIGHT = 500; // in pixels
    private double t;              // Simulation time
    private Timer pase;
    private double delta_t;        // in seconds
    private double refreshPeriod;  // in seconds
    private Vector2D g;
    private float viscosity;
    private PhysicsLabPanel labPanel;
    private Vector<PhysicsElement> elements;
    private boolean removing;

    public MyWorld() {
        t = 0;
        refreshPeriod = 0.06;      // 80 [ms]
        delta_t = 0.0001;          // 0.1 [ms]
        pase = new Timer((int) (refreshPeriod * 1000), this);
        g = new Vector2D(0, 0);
        viscosity = 0.0f;
        elements = new Vector<PhysicsElement>();
        labPanel = null;
        removing = false;
    }
/**Metodo que retorna verdadero en caso de estar corriendo el reloj y falso en caso contrario.*/
    public boolean isRunning() {
        return pase.isRunning();
    }

    public void setPanel(PhysicsLabPanel p) {
        labPanel = p;
    }

    public void setDelta_t(double delta) {
        delta_t = delta;
    }

    public void setRefreshPeriod(double rp) {
        refreshPeriod = rp;
        pase.setDelay((int) (refreshPeriod * 1000));
    }

    public void setGravity(Vector2D gravity) {
        g = gravity;
    }

    public void setViscosity(float v) {
        viscosity = v;
    }

    public void addElement(PhysicsElement e) {
        elements.add(e);
        labPanel.repaint();
    }
/**Metodo que llama a e.reset de cada elemento del mundo, para reiniciar la simulacion pero desde la posicion actual.*/
    public void start() {
        if (pase.isRunning()) {
            return;
        }
        for (PhysicsElement e : elements) {
            e.setInitialState(g, viscosity);
            e.reset();
        }
        labPanel.desableMouseListener();
        pase.start();
    }
/**Metodo Start() de la version anterior, se le modifico su nombre para adaptarse a los requerimientos del problema.*/
    public void resume() {
        if (pase.isRunning()) {
            return;
        }
        for (PhysicsElement e : elements) {
            e.setInitialState(g, viscosity);
        }
        labPanel.desableMouseListener();
        pase.start();
    }

    public void stop() {
        pase.stop();
        labPanel.enableMouseListener();
    }

    public void actionPerformed(ActionEvent event) {  // simulate passing time
        double nextStop = t + refreshPeriod;
        for (; t < nextStop; t += delta_t) {
            for (PhysicsElement e : elements) {
                e.computeNextState(delta_t, g, viscosity);
            }
            for (PhysicsElement e : elements) {
                e.updateState();
            }
        }
        labPanel.repaint();
    }

    public void paintComponent(Graphics2D g2d) {
        for (PhysicsElement e : elements) {
            if (e instanceof Drawable) {
                ((Drawable) e).draw(g2d);
            }
        }
    }

    public Drawable find(int x, int y) {
        for (PhysicsElement e : elements) {
            if (e instanceof Drawable) {
                Drawable de = (Drawable) e;
                if (de.contains(x, y)) {
                    return de;
                }
            }
        }
        return null;
    }

    public AttachableElement findAttachableElement(int x, int y) {
        for (PhysicsElement e : elements) {
            if (e instanceof AttachableElement) {
                AttachableElement ae = (AttachableElement) e;
                if (ae instanceof Drawable) {
                    if (((Drawable) ae).contains(x, y)) {
                        return ae;
                    }
                }
            }
        }
        return null;
    }
/**Metodo que llama labPanel y le pide que cambie su cursor.*/
    public void toggleRemoveElement() {
        removing = !removing;
        labPanel.toggleCursor();
    }
/**Metodo que retorna verdadero en caso de que se este removiendo algun objeto.*/
    public boolean isRemoving() {
        return removing;
    }
/**Metodo que remueve elemento del sistema, primero pregunta que tipo de elemento es y le pide a ese elemento que se desenganche de los elementos que lo rodean y luego elimina su referencia.*/
    public void removeElement(Drawable element) {

        if (element instanceof Spring) {
            Spring s = (Spring) element;
            s.detachAll();
        } else if (element instanceof FixedHook) {
            FixedHook f = (FixedHook) element;
            f.detachAll();
        } else if (element instanceof Ball) {
            Ball b = (Ball) element;
            b.detachAll();
        }
        toggleRemoveElement();
        elements.remove((PhysicsElement) element);
    }
}
