/**Se vario con respecto a la version anterior. Se anadieron 4 metodos, aEndIs(), bEndIs(), nullAEnd(), nullBEnd(), detachAll() y se implemento metodo reset()..*/
public class Spring extends PhysicsElement {

    private static int id = 0;  // Spring identification
    protected final float restLength;
    protected final float stiffness;
    protected AttachableElement a_end, b_end;

    protected Spring() {   // nobody can create a block without state
        super(id++);
        stiffness = restLength = 0;
    }

    public Spring(float restLength, float stiffness) {
        super(id++);
        this.restLength = restLength;
        this.stiffness = stiffness;
        a_end = b_end = null;
    }

    public void reset() {
    }
/**Metodo que retorna verdadero en caso de que el la referencia entregada como argumento sea igual a la referencia de la terminacion A del resorte.*/
    public boolean aEndIs(PhysicsElement element) {
        if (element == a_end) {
            return true;
        } else {
            return false;
        }
    }
/**Metodo que retorna verdadero en caso de que el la referencia entregada como argumento sea igual a la referencia de la terminacion B del resorte.*/
    public boolean bEndIs(PhysicsElement element) {
        if (element == b_end) {
            return true;
        } else {
            return false;
        }
    }

    public void attachAend(AttachableElement ae) {
        a_end = ae;
        ae.attachSpring(this);
    }

    public void detachAend() {
        if (a_end == null) {
            return;
        }
        a_end.detachSpring(this);
        a_end = null;
    }

    public void attachBend(AttachableElement ae) {
        b_end = ae;
        ae.attachSpring(this);
    }

    public void detachBend() {
        if (b_end == null) {
            return;
        }
        b_end.detachSpring(this);
        b_end = null;
    }
/**Metodo que anula la referencia de su terminacion A.*/
    public void nullAEnd() {
        a_end = null;
    }
/**Metodo que anula la referencia de su terminacion B.*/
    public void nullBEnd() {
        b_end = null;
    }
/**Metodo que le pide a sus 2 extremos que borren la referencia que tienen sobre el.*/
    public void detachAll() {
        detachAend();
        detachBend();
    }

    public String getStiffness() {
        String s = stiffness + "";
        return s;
    }

    public Vector2D getAendPosition() {
        if (a_end != null) {
            return a_end.getPosition();
        }
        if (b_end != null) {
            return b_end.getPosition().minus(new Vector2D(0, restLength));
        }
        return new Vector2D();
    }

    public Vector2D getBendPosition() {
        if (b_end != null) {
            return b_end.getPosition();
        }
        if (a_end != null) {
            return a_end.getPosition().plus(new Vector2D(0, restLength));
        }
        return new Vector2D(0, restLength);
    }

    protected Vector2D getVector() {
        return getBendPosition().minus(getAendPosition());
    }

    public Vector2D getForce(AttachableElement ae) {
        Vector2D force = new Vector2D();
        if ((a_end == null) || (b_end == null)) {
            return force;
        }
        if ((ae != a_end) && (ae != b_end)) {
            return force;
        }
        Vector2D springVector = getVector();
        double stretch = springVector.module() - restLength;
        force = springVector.unitary().times(stretch * stiffness);
        if (ae == a_end) {
            return force;
        }
        return force.times(-1.0);
    }

    public String getDescription() {
        return "Spring_" + getId() + ": " + Vector2D.getDescription() + " ," + Vector2D.getDescription();
    }

    public String getState() {
        String s = getAendPosition() + "," + getBendPosition();
        return s;
    }
}
